@@ -2,7 +2,7 @@ from lxml import etree as ET
 import os, shutil
 import logging
 import tempfile
-from . import static, utils
+from . import static, utils, generate
 from . import build as builder
 from .static.pretext import pretext as core
 from pathlib import Path
@@ -170,6 +170,8 @@ class Project():
                 targets_element.append(target.xml_element())
         self.__xml_element = xml_element
         self.__project_path = project_path
+        # prepre core PreTeXt pythons scripts
+        self.init_ptxcore()
 
     def xml_element(self):
         return self.__xml_element
@@ -194,25 +196,21 @@ class Project():
         else:
             return None
 
-    def view(self,target_name,access,port,watch=False,build=False):
+    def view(self,target_name,access,port,watch=False):
         target = self.target(target_name)
         directory = target.output_dir()
-        if watch or build:
-            log.info("Building target...")
-            self.build(target_name)
         if watch:
             watch_directory = target.source_dir()
         else:
             watch_directory = None
         if not utils.directory_exists(target.output_dir()):
-            log.error(f"The directory `{target.output_dir()}` does not exist. Maybe try `pretext build {target.name()}` first?")
+            log.error(f"The directory `{target.output_dir()}` does not exist.")
+            log.error(f"Run `pretext build {target}` to build your project before viewing.")
             return
         watch_callback=lambda:self.build(target_name)
         utils.run_server(directory,access,port,watch_directory,watch_callback)
 
-    def build(self,target_name,webwork=False,diagrams=False,diagrams_format='defaults',only_assets=False,clean=False):
-        # prepre core PreTeXt pythons scripts
-        self.init_ptxcore()
+    def build(self,target_name,clean=False):
         # Check for xml syntax errors and quit if xml invalid:
         if not self.xml_source_is_valid(target_name):
             return
@@ -239,66 +237,75 @@ class Project():
                 shutil.rmtree(target.output_dir())
         #if custom xsl, copy it into a temporary directory (different from the building temporary directory)
         custom_xsl = None
-        temp_xsl_dir = None
-        if target.xsl_path() is not None:
-            temp_xsl_dir = tempfile.mkdtemp()
-            log.info(f'Building with custom xsl {target.xsl_path()} specified in project.ptx')
-            utils.copy_fix_xsl(target.xsl_path(), temp_xsl_dir)
-            custom_xsl = os.path.join(temp_xsl_dir, os.path.basename(target.xsl_path()))
-        log.info(f"Preparing to build into {target.output_dir()}.")
-        #build targets:
-        if webwork:
-            # prepare params; for now assume only server is passed
-            # see documentation of pretext core webwork_to_xml
-            # handle this exactly as in webwork_to_xml (should this
-            # be exported in the pretext core module?)
-            webwork_output = os.path.join(target.generated_dir(),'webwork')
-            os.makedirs(webwork_output, exist_ok=True)
+        with tempfile.TemporaryDirectory() as temp_xsl_dir:
+            if target.xsl_path() is not None:
+                log.info(f'Building with custom xsl {target.xsl_path()} specified in project.ptx')
+                utils.copy_expanded_xsl(target.xsl_path(), temp_xsl_dir)
+                custom_xsl = os.path.join(temp_xsl_dir, os.path.basename(target.xsl_path()))
+            log.info(f"Preparing to build into {target.output_dir()}.")
             try:
-                server_url = target.stringparams()['server']
+                if (target.format()=='html' or target.format()=='html-zip'):
+                    zipped = (target.format()=='html-zip')
+                    builder.html(target.source(),target.publication(),target.output_dir(),target.stringparams(),custom_xsl,target.xmlid_root(),zipped)
+                elif target.format()=='latex':
+                    builder.latex(target.source(),target.publication(),target.output_dir(),target.stringparams(),custom_xsl)
+                    # core script doesn't put a copy of images in output for latex builds, so we do it instead here
+                    shutil.copytree(target.external_dir(),os.path.join(target.output_dir(),"external"),dirs_exist_ok=True)
+                    shutil.copytree(target.generated_dir(),os.path.join(target.output_dir(),"generated"),dirs_exist_ok=True)
+                elif target.format()=='pdf':
+                    builder.pdf(target.source(),target.publication(),target.output_dir(),target.stringparams(),custom_xsl,target.pdf_method())
             except Exception as e:
-                root_cause = str(e)
-                server_url = "https://webwork-ptx.aimath.org"
-                log.warning(f"No server name, {root_cause}.")
-                log.warning(f"Using default {server_url}")
-            builder.webwork(target.source(), target.publication(), webwork_output, target.stringparams(), server_url)
-        elif len(target.source_xml().xpath('//webwork[node()|@*]')) > 0:
-            log.warning(
-                "The source has WeBWorK exercises, but you are not re(processing) these.  Run `pretext build` with the `-w` flag if updates are needed.")
-        if diagrams:
-            builder.diagrams(
-                target.source(), target.publication(), target.generated_dir(), target.stringparams(), 
-                target.format(), diagrams_format, target.xmlid_root(), target.pdf_method(),
-            )
+                log.critical(
+                    f"A fatal error has occurred:\n {e} \nFor more info, run pretext with `-v debug`")
+                log.debug(f"Critical error info:\n****\n", exc_info=True)
+                return
+            # build was successful
+            log.info(f"\nSuccess! Run `pretext view {target.name()}` to see the results.\n")
+    
+    def generate(self,target_name,asset_list=None,all_formats=False):
+        if asset_list is None:
+            asset_list = []
+            gen_all = True
         else:
-            source_xml = target.source_xml()
-            if target.format()=="html" and len(source_xml.xpath('//asymptote|//latex-image|//sageplot')) > 0:
-                log.warning("The source has generated images (<latex-image/>, <asymptote/>, or <sageplot/>), "+
-                            "but these will not be (re)built. Run `pretext build` with the `-d` flag if updates are needed.")
-            if target.format()=="latex" and len(source_xml.xpath('//asymptote|//sageplot|//video[@youtube]|//interactive[not(@preview)]')) > 0:
-                log.warning("The source has interactive elements or videos that need a preview to be generated, "+
-                            "but these will not be (re)built. Run `pretext build` with the `-d` flag if updates are needed.")
-        try:
-            if (target.format()=='html' or target.format()=='html-zip') and not only_assets:
-                zipped = (target.format()=='html-zip')
-                builder.html(target.source(),target.publication(),target.output_dir(),target.stringparams(),custom_xsl,target.xmlid_root(),zipped)
-            elif target.format()=='latex' and not only_assets:
-                builder.latex(target.source(),target.publication(),target.output_dir(),target.stringparams(),custom_xsl)
-                # core script doesn't put a copy of images in output for latex builds, so we do it instead here
-                shutil.copytree(target.external_dir(),os.path.join(target.output_dir(),"external"),dirs_exist_ok=True)
-                shutil.copytree(target.generated_dir(),os.path.join(target.output_dir(),"generated"),dirs_exist_ok=True)
-            elif target.format()=='pdf' and not only_assets:
-                builder.pdf(target.source(),target.publication(),target.output_dir(),target.stringparams(),custom_xsl,target.pdf_method())
-        except Exception as e:
-            log.debug(f"Critical error info:\n", exc_info=True)
-            log.critical(
-                f"A fatal error has occurred:\n {e} \nFor more info, run pretext with `-v debug`")
-            return
-        # build was successful
-        log.info(f"\nSuccess! Run `pretext view {target.name()}` to see the results.\n")
-        # remove temporary xsl directory if it was created:
-        if (temp_xsl_dir and os.path.exists(temp_xsl_dir)):
-            shutil.rmtree(temp_xsl_dir)
+            gen_all = False
+        target = self.target(target_name)
+        os.makedirs(target.generated_dir(), exist_ok=True)
+        #build targets:
+        if gen_all or "webwork" in asset_list:
+            webwork_output = os.path.join(target.generated_dir(),'webwork')
+            generate.webwork(
+                target.source(), target.publication(), webwork_output, target.stringparams()
+            )
+        if gen_all or "latex-image" in asset_list:
+            generate.latex_image(
+                target.source(), target.publication(), target.generated_dir(), target.stringparams(), 
+                target.format(), target.xmlid_root(), target.pdf_method(), all_formats
+            )
+        if gen_all or "asymptote" in asset_list:
+            generate.asymptote(
+                target.source(), target.publication(), target.generated_dir(), target.stringparams(), 
+                target.format(), target.xmlid_root(), all_formats
+            )
+        if gen_all or "sageplot" in asset_list:
+            generate.sageplot(
+                target.source(), target.publication(), target.generated_dir(), target.stringparams(), 
+                target.format(), target.xmlid_root(), all_formats
+            )
+        if gen_all or "interactive" in asset_list:
+            generate.interactive(
+                target.source(), target.publication(), target.generated_dir(), target.stringparams(), 
+                target.xmlid_root(),
+            )
+        if gen_all or "youtube" in asset_list:
+            generate.youtube(
+                target.source(), target.publication(), target.generated_dir(), target.stringparams(), 
+                target.xmlid_root(),
+            )
+        if gen_all or "codelens" in asset_list:
+            generate.codelens(
+                target.source(), target.publication(), target.generated_dir(), target.stringparams(), 
+                target.xmlid_root(),
+            )
 
     def deploy(self,target_name,commit_message="Update to PreTeXt project source."):
         try:
@@ -306,6 +313,7 @@ class Project():
         except ImportError:
             log.error("Git must be installed to use this feature, but couldn't be found.")
             return
+        log.info("")
         target = self.target(target_name)
         if target.format() != "html":
             log.error("Only HTML format targets are supported.")
@@ -327,31 +335,8 @@ class Project():
                     w.set_value("user", "email", email)
             repo.git.add(all=True)
             repo.git.commit(message="Initial commit")
-            repo.heads.master.rename("main")
-            log.info("Success!")
-            log.info("")
-            log.info("If you haven't already, configure SSH with GitHub by following instructions at:")
-            log.info("    https://docs.github.com/en/authentication/connecting-to-github-with-ssh")
-            log.info("")
-            log.info("And if you haven't already, create a remote GitHub repository for this project at:")
-            log.info("    https://github.com/new")
-            log.info("(Do NOT check any \"initialize\" options.)")
-            log.info("Visit Settings > Pages on your new GitHub repository's page to enable GitHub Pages,")
-            log.info("selecting the `main` branch with the `/docs` folder.")
-            log.info("")
-            log.info("Then within your new repository homepage's \"Quick setup\" section select \"SSH\".")
-            log.info("And copy/paste your `git@github.com:YourUserName/repo-name.git` info below.")
-            ssh_info = input("Remote SSH info: ")
-            log.info("")
-            log.info("Attempting to connect to GitHub (your SSH password may be required)...")
-            try:
-                repo.create_remote("origin", url=ssh_info)
-                repo.git.push('--set-upstream', "origin", "main")
-            except git.exc.GitCommandError:
-                log.error(f"There was an issue connecting to GitHub via `{ssh_info}`")
-                log.error("Deploy was unsuccessful.")
-                return
-            log.info(f"Successfully connected to GitHub via `{ssh_info}`!")
+            repo.active_branch.rename("main")
+            log.info("Successfully initialized new Git repository!")
             log.info("")
         if repo.active_branch.name != "main":
             log.critical("The active Git branch is not set to `main`.")
@@ -368,9 +353,12 @@ class Project():
             log.error(f"Try running `pretext build {target.name()}` first.")
             return
         log.info(f"Preparing to deploy the latest build located in `{target.output_dir()}`.")
+        log.info("")
         docs_path = os.path.join(self.__project_path,"docs")
+        # copy latest build to docs
         shutil.rmtree(docs_path,ignore_errors=True)
         shutil.copytree(target.output_dir(),docs_path)
+        # add empty .nojekyll for ghpages
         with open(os.path.join(docs_path,'.nojekyll'), 'w') as fp:
             pass
         log.info(f"Latest build copied to `{docs_path}`.")
@@ -380,16 +368,61 @@ class Project():
             repo.git.commit(message=f"Deploy latest build of target {target.name()}.")
         except git.exc.GitCommandError:
             log.warning("Latest build was already committed by Git (but may not have been deployed).")
-            pass
-        log.info("Deploying to GitHub. (Your SSH password may be required.)")
+            log.info("")
         try:
-            repo.git.push()
+            origin = repo.remotes.origin
+        except AttributeError:
+            log.warning("Remote GitHub repository is not yet configured.")
+            log.info("")
+            log.info("And if you haven't already, create a remote GitHub repository for this project at:")
+            log.info("    https://github.com/new")
+            log.info("(Do NOT check any \"initialize\" options.)")
+            log.info("Then provide your GitHub info below:")
+            log.info("")
+            username = input("Your GitHub username (e.g. JaneDoe): ").strip()
+            reponame = input("Your GitHub repo name (e.g. my-great-book-repo): ").strip()
+            ssh_info = f"git@github.com:{username}/{reponame}.git"
+            repo.create_remote("origin", url=ssh_info)
+            origin = repo.remotes.origin
+            log.info("")
+        log.info(f"Attempting to connect to remote repository at `{origin.url}`...")
+        log.info("(Your SSH password may be required.)")
+        log.info("")
+        try:
+            repo_user = origin.url.split(":")[1].split("/")[0]
+            repo_name = origin.url.split(":")[1].split("/")[1][:-4]
+            repo_url = f"https://github.com/{repo_user}/{repo_name}/"
+            pages_url = f"https://{repo_user}.github.io/{repo_name}/"
+        except:
+            repo_url = f"(unable to find GitHub URL from {origin.url})"
+            pages_url = f"(unable to find GitHub Pages URL from {origin.url})"
+        try:
+            origin.push(refspec="main:main")
         except git.exc.GitCommandError:
-            log.error("Unable to push to GitHub. Try running `git push` yourself.")
+            log.warning(f"There was an issue connecting to GitHub repository located at {repo_url}")
+            log.info("")
+            log.info("If you haven't already, configure SSH with GitHub by following instructions at:")
+            log.info("    https://docs.github.com/en/authentication/connecting-to-github-with-ssh")
+            log.info("Then try to deploy again.")
+            log.info("")
+            log.info(f"If `{origin.url}` doesn't match your GitHub repository,")
+            log.info("use `git remote remove origin` on the command line then try to deploy again.")
+            log.info("")
+            log.error("Deploy was unsuccessful.")
             return
         log.info("")
-        log.info(f"Latest build successfully pushed to GitHub!")
-        log.info("Visit Settings > Pages on your GitHub repository's page to check deploy status.")
+        log.info("Latest build successfully pushed to GitHub!")
+        log.info("")
+        log.info("Visit")
+        log.info(f"    {repo_url}settings/pages")
+        log.info("to enable GitHub Pages, selecting the `main` branch with the `/docs` folder.")
+        log.info("")
+        log.info("Visit")
+        log.info(f"    {repo_url}deployments")
+        log.info("to check on the status of your GitHub Pages deployment.")
+        log.info("")
+        log.info("Your built project will soon be available to the public at:")
+        log.info(f"    {pages_url}")
 
     def xml_source_is_valid(self,target_name):
         target = self.target(target_name)
